@@ -54,17 +54,27 @@
 namespace massXpert
 {
 
+  enum formulaActions
+    {
+      ADD_FORMULA,
+      REMOVE_FORMULA,
+      CLEAR_WHOLE_MEMORY,
+      SIMPLIFY_FORMULA
+    };
+  
   CalculatorWnd::CalculatorWnd(QString &filePath)
   {
     m_forciblyClose = false;
     m_noDelistWnd = false;
 
+    m_formulaHandling = MXP_FORMULA_HANDLING_IMMEDIATE;
+    
     if (filePath.isEmpty())
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Polymer chemistry definition filepath empty."),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Polymer chemistry definition filepath empty."),
+			     QMessageBox::Ok);
       
 	return;
       }
@@ -74,23 +84,54 @@ namespace massXpert
     if (!initialize())
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Failed to initialize the calculator window."),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Failed to initialize the calculator window."),
+			     QMessageBox::Ok);
       }
 
     QSettings settings 
       (static_cast<Application *>(qApp)->configSettingsFilePath(), 
        QSettings::IniFormat);
     
-    settings.beginGroup("calculator_wnd");
+    QString section = QString ("calculator_wnd/%2")
+      .arg(m_polChemDef.name());
+    
+    settings.beginGroup(section);
     
     restoreGeometry(settings.value("geometry").toByteArray());
     
     m_ui.hSplitter->
       restoreState(settings.value("hSplitterSize").toByteArray());
     
+    int count = settings.beginReadArray("formulas");
+
+    for (int iter = 0; iter < count; ++iter) 
+      {
+	settings.setArrayIndex(iter);
+	
+	QString index;
+	index.setNum(iter);
+
+	QString value = settings.value(index).toString();
+	
+	m_ui.formulaComboBox->insertItem(iter, value);
+
+// 	qDebug () << __FILE__ << __LINE__
+// 		  << "Read line:" << value << "and" 
+// 		  << m_ui.formulaComboBox->itemText(iter);
+
+      }
+    
+    settings.endArray();
+
     settings.endGroup();
+
+    // But we want that the formulaComboBox line edit widget be empty
+    // even when formulas have been loaded. So clear the line edit
+    // widget of the combo box.
+
+    m_ui.formulaComboBox->lineEdit()->clear();
+
     
     show();
   }
@@ -109,14 +150,43 @@ namespace massXpert
       (static_cast<Application *>(qApp)->configSettingsFilePath(), 
        QSettings::IniFormat);
     
-    settings.beginGroup("calculator_wnd");
+    QString section = QString ("calculator_wnd/%2")
+      .arg(m_polChemDef.name());
+
+    settings.beginGroup(section);
     
     settings.setValue("geometry", saveGeometry());
     
     settings.setValue("hSplitterSize", m_ui.hSplitter->saveState());
     
+    settings.beginWriteArray("formulas");
+
+    int count = m_ui.formulaComboBox->count();
+    
+    for (int iter = 0; iter < count; ++iter)
+      {
+	settings.setArrayIndex(iter);
+
+	QString index;
+	index.setNum(iter);
+     
+	settings.setValue(index, m_ui.formulaComboBox->itemText(iter));
+
+// 	qDebug () << __FILE__ << __LINE__
+// 		  << "Set line:" << m_ui.formulaComboBox->itemText(iter);
+      }
+    
+    settings.endArray();
+    
     settings.endGroup();
     
+    // Force the writing of the geometry settings of the chemical pad
+    // (the settings are specific for each polymer chemistry
+    // definition). Look at the function to get it.
+
+    showChemPad(Qt::Checked);
+    showChemPad(Qt::Unchecked);
+    
 
     // We are asked not to remove this window from the list of all the
     // calculator windows. This occurs when all the windows are closed
@@ -159,70 +229,75 @@ namespace massXpert
     ////// Connection of the SIGNALS and SLOTS //////
 
     connect(m_ui.addToResultPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(addToResult()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(addToResult()));
   
     connect(m_ui.sendToResultPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(sendToResult()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(sendToResult()));
   
     connect(m_ui.removeFromResultPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(removeFromResult()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(removeFromResult()));
   
     connect(m_ui.clearSeedPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(clearSeed()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(clearSeed()));
   
     connect(m_ui.addToSeedPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(addToSeed()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(addToSeed()));
   
     connect(m_ui.sendToSeedPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(sendToSeed()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(sendToSeed()));
   
     connect(m_ui.removeFromSeedPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(removeFromSeed()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(removeFromSeed()));
   
     connect(m_ui.clearResultPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(clearResult()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(clearResult()));
   
     connect(m_ui.showRecorderCheckBox,
-	     SIGNAL(stateChanged(int)),
-	     this,
-	     SLOT(showRecorder(int)));
+	    SIGNAL(stateChanged(int)),
+	    this,
+	    SLOT(showRecorder(int)));
   
     connect(m_ui.showChemPadCheckBox,
-	     SIGNAL(stateChanged(int)),
-	     this,
-	     SLOT(showChemPad(int)));
+	    SIGNAL(stateChanged(int)),
+	    this,
+	    SLOT(showChemPad(int)));
   
     connect(m_ui.applyPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(apply()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(apply()));
 
     connect(m_ui.mzCalculationPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(mzCalculation()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(mzCalculation()));
 
     connect(m_ui.isotopicPatternCalculationPushButton,
-	     SIGNAL(clicked()),
-	     this,
-	     SLOT(isotopicPatternCalculation()));
+	    SIGNAL(clicked()),
+	    this,
+	    SLOT(isotopicPatternCalculation()));
 
+    connect(m_ui.formulaActionsComboBox,
+	    SIGNAL(activated(int)),
+	    this,
+	    SLOT(formulaActionsComboBoxActivated(int)));
+    
 
     m_ui.formulaSpinBox->setRange(-1000000000, 1000000000);
     m_ui.monomerSpinBox->setRange(-1000000000, 1000000000);
@@ -238,15 +313,31 @@ namespace massXpert
     if (!populatePolChemDefComboBoxes())
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Failed to populate the polymer"
-				  " chemistry data comboboxes.\n"
-				  "The calculator will not work properly."),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Failed to populate the polymer"
+				" chemistry data comboboxes.\n"
+				"The calculator will not work properly."),
+			     QMessageBox::Ok);
 
 	return false;
       }
     
+    // Populate the formulaActionsComboBox with the actions:
+
+
+    m_ui.formulaActionsComboBox->insertItem(ADD_FORMULA,
+					    tr("Add formula to memory"));
+    
+    m_ui.formulaActionsComboBox->insertItem(REMOVE_FORMULA,
+					    tr("Remove formula from memory"));
+    
+    m_ui.formulaActionsComboBox->insertItem(CLEAR_WHOLE_MEMORY,
+					    tr("Clear whole memory"));
+    
+    m_ui.formulaActionsComboBox->insertItem(SIMPLIFY_FORMULA,
+					    tr("Simplify formula"));
+    
+
     if (!setupChemicalPad())
       return false;
     
@@ -263,13 +354,19 @@ namespace massXpert
   }
 
 
+  QString
+  CalculatorWnd::polChemDefName()
+  {
+    return m_polChemDef.name();
+  }
+  
 
   void
   CalculatorWnd::updateWindowTitle()
   {
     setWindowTitle(tr("%1 %2[*]")
-		    .arg(tr("massXpert - Calculator:"))
-		    .arg(m_polChemDef.name()));
+		   .arg(tr("massXpert - Calculator:"))
+		   .arg(m_polChemDef.name()));
   }
 
 
@@ -349,15 +446,15 @@ namespace massXpert
     if (! file.open(QFile::ReadOnly)) 
       {
 	QMessageBox::information(0, 
-				  tr("massXpert - Calculator"),
-				  tr("Chemical pad config file not found.\n"
-				      "The chemical pad will not work properly."),
-				  QMessageBox::Ok);
+				 tr("massXpert - Calculator"),
+				 tr("Chemical pad config file not found.\n"
+				    "The chemical pad will not work properly."),
+				 QMessageBox::Ok);
 	return true;
       }
-  
-    bool ret = m_chemPadDlg.setup(filePath);
-  
+    
+    bool ret = m_chemPadDlg.setup(filePath, m_polChemDef.name());
+    
     return ret;
   }
 
@@ -434,9 +531,9 @@ namespace massXpert
     if (m_seedPonderable.mono() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed mono mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed mono mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -446,9 +543,9 @@ namespace massXpert
     if (m_seedPonderable.avg() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed avg mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed avg mass"),
+			     QMessageBox::Ok);
 	return;
       }
 
@@ -456,7 +553,7 @@ namespace massXpert
     // Prepare the recorder string.
   
     QString recorder = tr("Added seed(mono: %1 ; avg: %2) \n"
-			   "to result(was mono:%3 ; avg: %4) \n")
+			  "to result(was mono:%3 ; avg: %4) \n")
       .arg(m_seedPonderable.mono(locale))
       .arg(m_seedPonderable.avg(locale))
       .arg(m_resultPonderable.mono(locale))
@@ -496,9 +593,9 @@ namespace massXpert
     if (m_seedPonderable.mono() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed mono mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed mono mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -509,9 +606,9 @@ namespace massXpert
     if (m_seedPonderable.avg() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed avg mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed avg mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -519,7 +616,7 @@ namespace massXpert
     // Prepare the recorder string.
   
     QString recorder = tr("Sent seed(mono: %1 ; avg: %2) \n"
-			   "to result(was mono:%3 ; avg: %4) \n")
+			  "to result(was mono:%3 ; avg: %4) \n")
       .arg(m_seedPonderable.mono(locale))
       .arg(m_seedPonderable.avg(locale))
       .arg(m_resultPonderable.mono(locale))
@@ -555,9 +652,9 @@ namespace massXpert
     if (m_seedPonderable.mono() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed mono mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed mono mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -568,9 +665,9 @@ namespace massXpert
     if (m_seedPonderable.avg() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed avg mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed avg mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -578,7 +675,7 @@ namespace massXpert
     // Prepare the recorder string.
   
     QString recorder = tr("Removed seed(mono: %1 ; avg: %2) \n"
-			   "from result(was mono:%3 ; avg: %4) \n")
+			  "from result(was mono:%3 ; avg: %4) \n")
       .arg(m_seedPonderable.mono(locale))
       .arg(m_seedPonderable.avg(locale))
       .arg(m_resultPonderable.mono(locale))
@@ -627,9 +724,9 @@ namespace massXpert
     if (m_seedPonderable.mono() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed mono mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed mono mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -640,9 +737,9 @@ namespace massXpert
     if (m_seedPonderable.avg() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed avg mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed avg mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -650,7 +747,7 @@ namespace massXpert
     // Prepare the recorder string.
   
     QString recorder = tr("Added result(mono: %1 ; avg: %2) \n"
-			   "to seed(was mono:%3 ; avg: %4) \n")
+			  "to seed(was mono:%3 ; avg: %4) \n")
       .arg(m_resultPonderable.mono(locale))
       .arg(m_resultPonderable.avg(locale))
       .arg(m_seedPonderable.mono(locale))
@@ -681,7 +778,7 @@ namespace massXpert
     // Prepare the recorder string.
   
     QString recorder = tr("Sent result(mono: %1 ; avg: %2) \n"
-			   "to seed(was mono:%3 ; avg: %4) \n")
+			  "to seed(was mono:%3 ; avg: %4) \n")
       .arg(m_resultPonderable.mono(locale))
       .arg(m_resultPonderable.avg(locale))
       .arg(m_seedPonderable.mono(locale))
@@ -717,9 +814,9 @@ namespace massXpert
     if (m_seedPonderable.mono() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed mono mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed mono mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -730,9 +827,9 @@ namespace massXpert
     if (m_seedPonderable.avg() == 0 && !ok)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("Error with seed avg mass"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("Error with seed avg mass"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -740,7 +837,7 @@ namespace massXpert
     // Prepare the recorder string.
   
     QString recorder = tr("Removed result(mono: %1 ; avg: %2) \n"
-			   "from seed(was mono:%3 ; avg: %4) \n")
+			  "from seed(was mono:%3 ; avg: %4) \n")
       .arg(m_resultPonderable.mono(locale))
       .arg(m_resultPonderable.avg(locale))
       .arg(m_seedPonderable.mono(locale))
@@ -772,20 +869,21 @@ namespace massXpert
 
 
   void 
-  CalculatorWnd::apply()
+  CalculatorWnd::apply(const QString &formula)
   {
     Application *application = static_cast<Application *>(qApp);
     QLocale locale = application->locale();
     QString mass;
     bool ok = true;
     bool calculationPerformed = false;
-  
+    int res = -1;
+    
     if (m_polChemDef.atomList().size() == 0)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("No atom list is available"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("No atom list is available"),
+			     QMessageBox::Ok);
 	return;
       }
   
@@ -825,7 +923,7 @@ namespace massXpert
 	// We use the m_tempPonderable as a repository for the masses
 	// actually used for the computation, be them seed or result:
 
-	if(m_ui.seedMonoMassLineEdit->text().isEmpty()
+	if (m_ui.seedMonoMassLineEdit->text().isEmpty()
 	    || m_ui.seedAvgMassLineEdit->text().isEmpty())
 	  {
 	    // The seed masses are empty and the result masses
@@ -843,9 +941,9 @@ namespace massXpert
 	    if (m_tempPonderable.avg() == 0 && !ok)
 	      {
 		QMessageBox::warning(0, 
-				      tr("massXpert - Calculator"),
-				      tr("Error with seed avg mass"),
-				      QMessageBox::Ok);
+				     tr("massXpert - Calculator"),
+				     tr("Error with seed avg mass"),
+				     QMessageBox::Ok);
 		return;
 	      }
       
@@ -855,9 +953,9 @@ namespace massXpert
 	    if (m_tempPonderable.avg() == 0 && !ok)
 	      {
 		QMessageBox::warning(0, 
-				      tr("massXpert - Calculator"),
-				      tr("Error with seed avg mass"),
-				      QMessageBox::Ok);
+				     tr("massXpert - Calculator"),
+				     tr("Error with seed avg mass"),
+				     QMessageBox::Ok);
 		return;
 	      }
 	  }
@@ -873,30 +971,99 @@ namespace massXpert
     // Prepare the recorder string.  
     QString recorder = 
       tr("===================================================================\n"
-	  "Seed masses: %1 -- %2\n")
+	 "Seed masses: %1 -- %2\n")
       .arg(m_seedPonderable.mono())
       .arg(m_seedPonderable.avg());
   
     // And now append the new recorder string to the recorder textEdit.
     m_recorderDlg.record(recorder);
     
-    // Finally we can start doing work with the chemical entities...
 
-    // Atoms/Formulae
-    int res = accountFormula();
-  
-    if (res == -1)
-      QMessageBox::warning(0, 
-			    tr("massXpert - Calculator"),
-			    tr("Error accounting formula"),
-			    QMessageBox::Ok);
-    else if (res == 1)
+    // Finally we can start doing work with the chemical entities...
+    
+    // Formulae
+    
+    // If the 'formula' parameter contains a string, then two cases arise:
+
+    // 1. If m_formulaHandling has the MXP_FORMULA_HANDLING_PRINT bit
+    // set, then do not account the formula, but simply insert it in
+    // the formula line edit widget.
+
+    // 2. If m_formulaHandling has the MXP_FORMULA_HANDLING_WITH_SPACE
+    // bit set, then also add a space to the formula (this is
+    // interesting when sending a large number of forumulas, so that
+    // each sent one is distinct from the next one.
+
+    // 2. If m_formulaHandling has the MXP_FORMULA_HANDLING_IMMEDIATE
+    // bit set, then account the formula immediately and return.
+
+    // Both situations 1. and 2. are related to this function being
+    //  called from one chemical pad button.
+
+    if (!formula.isEmpty())
       {
-	recordResult();
-      
-	calculationPerformed = true;
+	// There is a formula in the parameter... Check if we have to
+	// evaluate it immediately or if we have to just insert it in
+	// the current formula comboBox widget's item.
+
+	if (m_formulaHandling & MXP_FORMULA_HANDLING_PRINT)
+	  {
+	    // Get the line edit widget of the comboBox.
+	    QLineEdit* lineEdit = m_ui.formulaComboBox->lineEdit();
+		
+	    if (m_formulaHandling & MXP_FORMULA_HANDLING_WITH_SPACE)
+	      {
+		lineEdit->insert(QString (" "));
+		lineEdit->insert(formula);
+		lineEdit->insert(QString (" "));
+	      }
+	    else
+	      {
+		lineEdit->insert(formula);
+	      }
+	    
+	    return;
+	  }
+	else
+	  {
+	    int res = accountFormula(formula);
+	    
+	    if (res == -1)
+	      QMessageBox::warning(0, 
+				   tr("massXpert - Calculator"),
+				   tr("Error accounting formula '%1'")
+				   .arg(formula),
+				   QMessageBox::Ok);
+	    else if (res == 1)
+	      {
+		recordResult();
+		
+		updateSeedResultLineEdits();
+	      }
+	    
+	    return;
+	  }
       }
-  
+    else
+      {
+	// No formula was put into the parameter in the call to this
+	// function. Just do the whole business as usual.
+
+	res = accountFormula();
+	
+	if (res == -1)
+	  QMessageBox::warning(0, 
+			       tr("massXpert - Calculator"),
+			       tr("Error accounting formula"),
+			       QMessageBox::Ok);
+	else if (res == 1)
+	  {
+	    recordResult();
+	    
+	    calculationPerformed = true;
+	  }
+      }
+      
 
     if (!m_polChemDef.name().isEmpty())
       {
@@ -905,11 +1072,11 @@ namespace massXpert
 
 	res = accountMonomer();
       
-	if(res == -1)
+	if (res == -1)
 	  QMessageBox::warning(0, 
-				tr("massXpert - Calculator"),
-				tr("Error accounting monomer"),
-				QMessageBox::Ok);
+			       tr("massXpert - Calculator"),
+			       tr("Error accounting monomer"),
+			       QMessageBox::Ok);
 	else if (res == 1)
 	  {
 	    recordResult();
@@ -918,13 +1085,13 @@ namespace massXpert
 	  }
       
       
-	res = accountModif();
+	res = accountModif ();
       
-	if(res == -1)
+	if (res == -1)
 	  QMessageBox::warning(0, 
-				tr("massXpert - Calculator"),
-				tr("Error accounting modif"),
-				QMessageBox::Ok);
+			       tr("massXpert - Calculator"),
+			       tr("Error accounting modif"),
+			       QMessageBox::Ok);
 	else if (res == 1)
 	  {
 	    recordResult();
@@ -935,11 +1102,11 @@ namespace massXpert
 
 	res = accountSequence();
       
-	if(res == -1)
+	if (res == -1)
 	  QMessageBox::warning(0, 
-				tr("massXpert - Calculator"),
-				tr("Error accounting sequence"),
-				QMessageBox::Ok);
+			       tr("massXpert - Calculator"),
+			       tr("Error accounting sequence"),
+			       QMessageBox::Ok);
 	else if (res == 1)
 	  {
 	    recordResult();
@@ -963,10 +1130,13 @@ namespace massXpert
   void
   CalculatorWnd::mzCalculation()
   {
+    // Call the constructor with mono/avg seeds.
     MzCalculationDlg *dlg = 
       new MzCalculationDlg(this,
-			    &m_polChemDef,
-			    &m_polChemDef.ionizeRule());
+			   &m_polChemDef,
+			   &m_polChemDef.ionizeRule(),
+			   m_resultPonderable.mono(),
+			   m_resultPonderable.avg());
   
     dlg->show();
   }
@@ -982,6 +1152,38 @@ namespace massXpert
   }
 
 
+  void
+  CalculatorWnd::formulaActionsComboBoxActivated(int index)
+  {
+    if (index == ADD_FORMULA)
+      {
+	//	qDebug() << __FILE__ << __LINE__ << "ADD_TO_MEMORY";
+	addFormulaToMemory();
+      }
+    
+    
+    if (index == REMOVE_FORMULA)
+      {
+	//	qDebug() << __FILE__ << __LINE__ << "ADD_TO_MEMORY";
+	removeFormulaFromMemory();
+      }
+    
+    
+    if (index == CLEAR_WHOLE_MEMORY)
+      {
+	//	qDebug() << __FILE__ << __LINE__ << "CLEAR_MEMORY";
+	clearWholeMemory();
+      }
+    
+    
+    if (index == SIMPLIFY_FORMULA)
+      {
+	//	qDebug() << __FILE__ << __LINE__ << "SIMPLIFY_FORMULA";
+	simplifyFormula();
+      }
+  }
+  
+
   int
   CalculatorWnd::accountFormula(const QString &formula, int times) 
   {
@@ -999,25 +1201,37 @@ namespace massXpert
     if (m_polChemDef.atomList().size() == 0)
       {
 	QMessageBox::warning(0, 
-			      tr("massXpert - Calculator"),
-			      tr("No atom list is available"),
-			      QMessageBox::Ok);
+			     tr("massXpert - Calculator"),
+			     tr("No atom list is available"),
+			     QMessageBox::Ok);
 	return false;
       }
   
     if (formula.isEmpty())
       {
 	// No string passed as parameter, get the formula from its
-	// corresponding lineEdit.
-	text = m_ui.formulaLineEdit->text();
-      
-	if(text.isEmpty())
+	// corresponding comboBox.
+	text =  m_ui.formulaComboBox->currentText();
+	      
+	if (text.isEmpty())
 	  return 0;
+
+	// We want to remove spaces from the text. This is useful so
+	// the user might enter spaces in the formula to ease
+	// re-reading the formula text (useful when entering complex
+	// formula for saccharides, for example, in my own
+	// experience).
+	text.remove(QRegExp("\\s+"));
     
+	// Note that we might also have a title for the formula, which
+	// is enclosed between `"' at the beginning of the line.
+
+	text.remove(QRegExp("\".*\""));
+	
 	// How many times should the formula be accounted for?
 	count = m_ui.formulaSpinBox->value();
       
-	if(count == 0)
+	if (count == 0)
 	  return 0;
       }
     else
@@ -1042,8 +1256,8 @@ namespace massXpert
     m_recorderDlg.record(recorder);
   
     if (!fake.accountMasses(&m_tempPonderable.rmono(),
-			     &m_tempPonderable.ravg(),
-			     count))
+			    &m_tempPonderable.ravg(),
+			    count))
       return -1;
   
     return 1;
@@ -1074,8 +1288,8 @@ namespace massXpert
   
     int index = -1;
     index = Monomer::isNameInList(text, 
-				   m_polChemDef.monomerList(),
-				   &monomer);
+				  m_polChemDef.monomerList(),
+				  &monomer);
   
     // It cannot be that the text in the combobox does not correspond to
     // a known monomer.
@@ -1091,8 +1305,8 @@ namespace massXpert
     m_recorderDlg.record(recorder);
   
     if (!monomer.accountMasses(&m_tempPonderable.rmono(),
-				&m_tempPonderable.ravg(),
-				count))
+			       &m_tempPonderable.ravg(),
+			       count))
       return -1;
   
     return 1;
@@ -1100,7 +1314,7 @@ namespace massXpert
 
 
   int
-  CalculatorWnd::accountModif()
+  CalculatorWnd::accountModif ()
   {
     Application *application = static_cast<Application *>(qApp);
     QLocale locale = application->locale();
@@ -1118,12 +1332,12 @@ namespace massXpert
   
     // Locate the reference modif and make a copy of it, as we may not
     // use itself, as the parsing operation do modify the object.
-    Modif modif(&m_polChemDef, "NOT_SET");
+    Modif modif (&m_polChemDef, "NOT_SET");
     int index = -1;
 
     index = Modif::isNameInList(text, 
-				 m_polChemDef.modifList(),
-				 &modif);
+				m_polChemDef.modifList(),
+				&modif);
   
     // It cannot be that the text in the combobox does not correspond to
     // a known modif.
@@ -1139,8 +1353,8 @@ namespace massXpert
     m_recorderDlg.record(recorder);
   
     if (!modif.accountMasses(&m_tempPonderable.rmono(),
-			      &m_tempPonderable.ravg(),
-			      count))
+			     &m_tempPonderable.ravg(),
+			     count))
       return -1;
   
     return 1;
@@ -1187,9 +1401,9 @@ namespace massXpert
 	const Monomer *monomer = sequence.at(iter);
 	Q_ASSERT(monomer);
       
-	if(!monomer->accountMasses(&m_tempPonderable.rmono(),
-				     &m_tempPonderable.ravg(),
-				     count))
+	if (!monomer->accountMasses(&m_tempPonderable.rmono(),
+				    &m_tempPonderable.ravg(),
+				    count))
 	  return -1;
       }
   
@@ -1203,18 +1417,18 @@ namespace massXpert
     if (state == Qt::Checked)
       {
 	QSettings settings 
-     (static_cast<Application *>(qApp)->configSettingsFilePath(), 
-       QSettings::IniFormat);
+	  (static_cast<Application *>(qApp)->configSettingsFilePath(), 
+	   QSettings::IniFormat);
 	m_recorderDlg.restoreGeometry 
-	 (settings.value("calculator_recorder_dlg/geometry").toByteArray());
+	  (settings.value("calculator_recorder_dlg/geometry").toByteArray());
 
 	m_recorderDlg.show();
       }
     else
       {
 	QSettings settings 
-     (static_cast<Application *>(qApp)->configSettingsFilePath(), 
-       QSettings::IniFormat);
+	  (static_cast<Application *>(qApp)->configSettingsFilePath(), 
+	   QSettings::IniFormat);
 	settings.setValue("calculator_recorder_dlg/geometry", 
 			  m_recorderDlg.saveGeometry());
       
@@ -1228,23 +1442,152 @@ namespace massXpert
     if (state == Qt::Checked)
       {
 	QSettings settings 
-     (static_cast<Application *>(qApp)->configSettingsFilePath(), 
-       QSettings::IniFormat);
-	m_chemPadDlg.restoreGeometry 
-	 (settings.value("calculator_chem_pad_dlg/geometry").toByteArray());
-
+	  (static_cast<Application *>(qApp)->configSettingsFilePath(), 
+	   QSettings::IniFormat);
+	
+	settings.beginGroup("calculator_chem_pad_dlg");
+	
+	QString setting = QString("%1-%2")
+	  .arg(polChemDefName())
+	  .arg("geometry");
+	
+	m_chemPadDlg.restoreGeometry(settings.value(setting).toByteArray());
+	
+	settings.endGroup();
+	
 	m_chemPadDlg.show();
       }
     else
       {
 	QSettings settings 
-     (static_cast<Application *>(qApp)->configSettingsFilePath(), 
-       QSettings::IniFormat);
-	settings.setValue("calculator_chem_pad_dlg/geometry", 
-			  m_chemPadDlg.saveGeometry());
-      
+	  (static_cast<Application *>(qApp)->configSettingsFilePath(), 
+	   QSettings::IniFormat);
+	
+	settings.beginGroup("calculator_chem_pad_dlg");
+	
+	QString setting = QString("%1-%2")
+	  .arg(polChemDefName())
+	  .arg("geometry");
+	
+	settings.setValue(setting, m_chemPadDlg.saveGeometry());
+	
+	settings.endGroup();
+	      
 	m_chemPadDlg.hide();
       }
   }
 
+
+  void  
+  CalculatorWnd::setFormulaHandling(int val)
+  {
+    m_formulaHandling = val;
+  }
+  
+
+  void 
+  CalculatorWnd::addFormulaToMemory()
+  {
+    // Get the current text from the comboBox's lineEdit widget.
+
+    QString text = m_ui.formulaComboBox->currentText();
+    
+    //    qDebug() << __FILE__ << __LINE__ << "text:" << text;
+    
+
+    if (text.isEmpty())
+      return;
+    
+    m_ui.formulaComboBox->addItem(text);
+  }
+  
+  
+  void 
+  CalculatorWnd::removeFormulaFromMemory()
+  {
+    int currentIndex = m_ui.formulaComboBox->currentIndex();
+    
+    if (currentIndex != -1)
+      m_ui.formulaComboBox->removeItem (currentIndex);
+  }
+  
+  
+  void 
+  CalculatorWnd::clearWholeMemory()
+  {
+    m_ui.formulaComboBox->clear();
+  }
+  
+  
+  void
+  CalculatorWnd::simplifyFormula()
+  {
+    // Imagine we do lots of formula stuff, and we want to refactor a
+    // given series of formulas into one single formula like this:
+
+    // We have +CH3 -C +C2H6O4N5P
+
+    // We want to have the result of this: that is C2H8O4N5P
+    
+    // To do the simplification we need the list of atoms of the
+    // current polymer chemistry definition.
+
+    if (m_polChemDef.atomList().size() == 0)
+      {
+	QMessageBox::warning(0, 
+			     tr("massXpert - Calculator"),
+			     tr("No atom list is available"),
+			     QMessageBox::Ok);
+	return;
+      }
+  
+    QString text = m_ui.formulaComboBox->currentText();
+    
+    if (text.isEmpty())
+      return;
+    
+    // We want to remove spaces from the text. This is useful so the
+    // user might enter spaces in the formula to ease re-reading the
+    // formula text (useful when entering complex formula for
+    // saccharides, for example, in my own experience).
+    text.remove(QRegExp("\\s+"));
+    
+    // Note that we might also have a title for the formula, which is
+    // enclosed between `"' at the beginning of the line.
+
+    text.remove(QRegExp("\".*\""));
+
+    // Validate the formula by creating a fake modification object
+    // with it and by accounting masses for that formula.
+
+    Modif fake(&m_polChemDef, "NOT_SET", text);
+    if (!fake.calculateMasses())
+      {
+	QMessageBox::warning(0, 
+			     tr("massXpert - Calculator"),
+			     tr("Failed to parse the formula."),
+			     QMessageBox::Ok);
+	
+	return;
+      }
+    
+    QString simplified = fake.simplify();
+
+    //    qDebug() << "simplified:" << simplified;
+
+    if (!simplified.isEmpty())
+      {
+	// Store the formula to the memory, so that the user can check
+	// things.
+
+	addFormulaToMemory();
+	
+	// Set the new simplified formula to the comboBox.
+	m_ui.formulaComboBox->lineEdit()->setText(simplified);
+      }
+    
+    return;
+  }
+  
+
 } // namespace massXpert
