@@ -38,6 +38,7 @@
 /////////////////////// Qt includes
 #include <QMessageBox>
 #include <QFileDialog>
+#include <QDebug>
 
 
 /////////////////////// Local includes
@@ -51,24 +52,24 @@ namespace massXpert
   CleavageDlg::CleavageDlg(QWidget *parent,
 			    Polymer *polymer,
 			    const PolChemDef *polChemDef,
-			    const CalcOptions *calcOptions,
+			    const CalcOptions &calcOptions,
 			    const IonizeRule *ionizeRule)
     : QDialog(parent),
       mp_polymer(polymer), 
       mp_polChemDef(polChemDef), 
-      mp_calcOptions(calcOptions),
+      m_calcOptions(calcOptions),
       mp_ionizeRule(ionizeRule)
   {
     Q_ASSERT(parent);
-    Q_ASSERT(!mp_polymer.isNull() && mp_polChemDef && 
-	      mp_calcOptions && mp_ionizeRule);
+    Q_ASSERT(!mp_polymer.isNull() && mp_polChemDef && mp_ionizeRule);
 
     m_ui.setupUi(this);
   
     mp_editorWnd = static_cast<SequenceEditorWnd *>(parent);
 
-    populateCleaveSpecComboBox();
-
+    populateCleaveSpecListWidget();
+    populateSelectedOligomerData();
+    
     setupTreeView();
     
     m_ui.leftCapPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
@@ -193,8 +194,140 @@ namespace massXpert
   }
 
 
+bool
+  CleavageDlg::populateSelectedOligomerData() 
+  {
+    CoordinateList coordList;
+    Coordinates *coordinates = 0;
+
+    if (!mp_editorWnd->mpa_editorGraphicsView->selectionIndices(&coordList) )
+      {
+	// We get the selection indices corresponding to the virtual
+	// selection, that is the sequence region encompassing the
+	// first residue of the sequence up to the monomer left of
+	// point.
+
+	// In this case we want the cleavage to be performed on the
+	// whole polymer sequence. Set its coordinates to the list of
+	// coordinates.
+
+	coordinates = new Coordinates(0, mp_polymer->size() - 1);
+
+	// Set the newly created coordinates to the coordinates list
+	// that we'll feed into the calculation options.
+
+	coordList.setCoordinates(*coordinates);
+	
+	// We do not need the coordinates object anymore, delete it.
+	delete coordinates;
+	coordinates = 0;
+      }
+
+    if (coordList.size() > 1)
+      {
+	QMessageBox::information(this, tr("massxpert - Polymer Cleavage"),
+				  tr("Cleavage simulations with\n"
+				      "multi-region selection are not" 
+				      "supported."),
+				  QMessageBox::Ok);
+
+	// Set the selection data as if all of the sequence was to be
+	// cleaved.
+	
+	coordinates = new Coordinates(0, mp_polymer->size() - 1);
+
+	// Set the newly created coordinates to the coordinates list
+	// that we'll feed into the calculation options.
+
+	coordList.setCoordinates(*coordinates);
+	
+	// We do not need the coordinates object anymore, delete it.
+	delete coordinates;
+	coordinates = 0;
+      }
+
+    // Now, if the checkbox asking for whole sequence cleavage is
+    // checked, then we have to make the cleavage on the whole
+    // sequence.
+    if (m_ui.wholeSequenceCheckBox->isChecked())
+      {
+	// The cleavage is to be performed on the whole polymer
+	// sequence.
+
+	coordinates = new Coordinates(0, mp_polymer->size() - 1);
+
+	// Set the newly created coordinates to the coordinates list
+	// that we'll feed into the calculation options.
+
+	coordList.setCoordinates(*coordinates);
+	
+	// We do not need the coordinates object anymore, delete it.
+	delete coordinates;
+	coordinates = 0;
+      }
+    
+    m_calcOptions.setCoordinateList(coordList);
+  
+    coordinates = coordList.last();
+    
+    m_ui.oligomerStartLabel->
+      setText(QString().setNum(coordinates->start() + 1));
+    
+    m_ui.oligomerEndLabel->
+      setText(QString().setNum(coordinates->end() + 1));
+
+
+    // We have to count the incomplete cross-links.
+
+    const CrossLinkList &crossLinkList = mp_polymer->crossLinkList();
+	  
+    int crossLinkPartial = 0;
+	  
+    for (int iter = 0; iter < crossLinkList.size(); ++iter)
+      {
+	CrossLink *crossLink = crossLinkList.at(iter);
+	  
+	int ret = crossLink->encompassedBy(coordList);
+	  
+	if(ret == MXP_CROSS_LINK_ENCOMPASSED_FULL)
+	  {
+	    // 		qDebug() << __FILE__ << __LINE__
+	    // 			  << "CrossLink at iter:" << iter
+	    // 			  << "is fully encompassed";
+	  }
+	else if (ret == MXP_CROSS_LINK_ENCOMPASSED_PARTIAL)
+	  {
+	    // 		qDebug() << __FILE__ << __LINE__
+	    // 			  << "CrossLink at iter:" << iter
+	    // 			  << "is partially encompassed";
+	      
+	    ++crossLinkPartial;
+	  }
+	else
+	  {
+	    // 		qDebug() << __FILE__ << __LINE__
+	    // 			  << "CrossLink at iter:" << iter
+	    // 			  << "is not encompassed at all";
+	  }
+      }
+
+    if (crossLinkPartial)
+      {
+	// Alert the user on the fact that the currently selected
+	// region does not encompass all of the cross-linked material.
+
+	QMessageBox::information(this, 
+				 tr("massXpert - Polymer Cleavage"),
+				 tr("There are incomplete cross-links"),
+				 QMessageBox::Ok);
+      }
+
+    return true;
+  }
+  
+
   void
-  CleavageDlg::populateCleaveSpecComboBox()
+  CleavageDlg::populateCleaveSpecListWidget()
   {
     PolChemDef *polChemDef = mp_editorWnd->polChemDef();
     Q_ASSERT(polChemDef);
@@ -204,7 +337,7 @@ namespace massXpert
 	CleaveSpec *cleaveSpec = polChemDef->cleaveSpecList().at(iter);
 	Q_ASSERT(cleaveSpec);
       
-	m_ui.cleavingAgentsComboBox->addItem(cleaveSpec->name());
+	m_ui.cleavageAgentListWidget->addItem(cleaveSpec->name());
       }
   
     return;
@@ -238,8 +371,12 @@ namespace massXpert
   void 
   CleavageDlg::cleave()
   {
+    // First update the selection data from the sequence editor window.
+    if (!populateSelectedOligomerData())
+      return;
+    
     // What's the cleavage agent ?
-    int value = m_ui.cleavingAgentsComboBox->currentIndex();
+    int value = m_ui.cleavageAgentListWidget->currentRow();
 
     CleaveSpec *cleaveSpec = mp_polChemDef->cleaveSpecList().at(value);
     
@@ -259,7 +396,7 @@ namespace massXpert
     mpa_cleaver = 0;
     
     mpa_cleaver= new Cleaver(mp_polymer, mp_polChemDef, cleaveOptions,
-			      *mp_calcOptions, *mp_ionizeRule);
+			     m_calcOptions, *mp_ionizeRule);
 
     mpa_cleaver->setOligomerList(&oligomerList);
     
@@ -323,34 +460,34 @@ namespace massXpert
   void
   CleavageDlg::updateCleavageDetails()
   {
-    if (mp_calcOptions->capping() & MXT_CAP_LEFT)
+    if (m_calcOptions.capping() & MXT_CAP_LEFT)
       m_ui.leftCapPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
     else
       m_ui.leftCapPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
 
-    if (mp_calcOptions->capping() & MXT_CAP_RIGHT)
+    if (m_calcOptions.capping() & MXT_CAP_RIGHT)
       m_ui.rightCapPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
     else
       m_ui.rightCapPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
 
 
-    if (mp_calcOptions->monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
+    if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
       m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
     else
       m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
 
-    if (mp_calcOptions->monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
+    if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
       m_ui.crossLinksPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
     else
       m_ui.crossLinksPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
 
 
-    if (mp_calcOptions->polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
+    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
       m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
     else
       m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
 
-    if (mp_calcOptions->polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
+    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
       m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
     else
       m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
