@@ -235,7 +235,7 @@ namespace massXpert
       {
 	QMessageBox::information(this, tr("massxpert - Fragmentation"),
 				  tr("Fragmentation simulations with\n"
-				      "multi-region selection is not " 
+				      "multi-region selection are not " 
 				      "supported."),
 				  QMessageBox::Ok);
 	return false;
