@@ -63,11 +63,11 @@ namespace massXpert
   
     mp_editorWnd = static_cast<SequenceEditorWnd *>(parent);
   
-    updateSelectionData();
-    if (m_calcOptions.coordinateList().size() > 1)
-      m_ui.currentSelectionRadioButton->setChecked(true);
-
-
+    // Set the current whole/selected sequence data in the
+    // m_calcOptions instance and in the corresponding widgets (true =
+    // read from sequence editor window).
+    updateSelectionData(true);
+    
     m_monoTolerance = 0;
     m_avgTolerance = 0;
   
@@ -210,7 +210,7 @@ namespace massXpert
     connect(m_ui.updateCurrentSelectionDataPushButton,
 	     SIGNAL(clicked()),
 	     this,
-	     SLOT(updateSelectionData()));
+	     SLOT(updateWholeSelectedSequenceData()));
 
 
     // MONO set of filtering widgets
@@ -330,12 +330,58 @@ namespace massXpert
     return true;
   }
 
+  void
+  MassSearchDlg::updateWholeSelectedSequenceData()
+  {
+    updateSelectionData(true);
+  }
+  
 
   bool 
-  MassSearchDlg::updateSelectionData()
+  MassSearchDlg::updateSelectionData(bool fromSequenceEditor)
   {
-    // There might be more than one region selections. So get all
-    // these coordinates !
+    // There are two situations: 
+
+    // 1. This function is called from the constructor to setup data
+    // in the whole/selected sequence groupBox widget. It gets the
+    // info directly from the sequence editor window
+    // (fromSequenceEditor = true). By default the param is true,
+    // because this is expected for when this slot is called upon
+    // clicking on the Update button.
+
+    // 2. This function is called only to set the whole/selected
+    // sequence data in the calculation options instance. The data are
+    // read directly from the widgets and the sequence editor window
+    // is not probed. 
+
+    if (!fromSequenceEditor)
+      {
+	// Only update that whole/selected sequence data in the
+	// m_calcOptions from the widgettry and not from the
+	// sequence. Because the user cannot modify the selection text
+	// in the line edit widget, all we need to know is: whole or
+	// selected sequence.
+
+	if (m_ui.wholeSequenceRadioButton->isChecked())
+	  {
+	    // Set the "selection" to be the whole sequence.
+	    
+	    Coordinates coordinates(0, mp_polymer->size() - 1);
+	    m_calcOptions.setCoordinateList(coordinates);
+	    
+	    m_ui.wholeSequenceRadioButton->setChecked(true);
+
+	    return true;
+	  }
+	// Else, we do not need to do anything. The selection is
+	// updated by clicking onto the Update button.
+
+	return true;
+      }
+
+    // At this point, we are asked to update the whole/selected
+    // sequence data directly from the sequence editor window. So get
+    // all these coordinates !
 	
     CoordinateList coordList;
 
@@ -384,6 +430,8 @@ namespace massXpert
 	    
 	    m_calcOptions.setCoordinateList(*coordinates);
 	  }
+
+	m_ui.currentSelectionRadioButton->setChecked(true);
       }
     else
       {
@@ -739,9 +787,10 @@ namespace massXpert
     // Update the options from the parent window.
     m_calcOptions = mp_editorWnd->calcOptions();
     
-    // Update the currently selected region(s) if any in the sequence
-    // editor.
-    updateSelectionData();
+    // Set the current whole/selected sequence data in the
+    // m_calcOptions instance (false = no read from sequence editor
+    // window).
+    updateSelectionData(false);
     
     updateIonizationData();
   
