@@ -339,7 +339,7 @@ namespace massXpert
 	    // The monomer(quintptr) pointer is the third string in the list.
 	    ok = false;
 	    quintptr pointerCastToInt = stringList.at(2).toInt(&ok);      
-	    Monomer *monomer =(Monomer *) pointerCastToInt;
+	    Monomer *monomer = reinterpret_cast<Monomer *>(pointerCastToInt);
 
 	    // Sanity check, are we dealing with the same monomer now
 	    // compared to the one of which the item was displayed when
@@ -770,13 +770,12 @@ namespace massXpert
     // End of 
     // for (int iter = 0; iter < m_indicesList.size(); ++iter)
 
-    // At this point we can delete the modif template.
-
-    delete modif;
-  
     m_ui.messagesTextEdit->append(tr("New operation: modify with %1\n")
-				   .arg(modif->name()) + errorList.join(""));
-
+				  .arg(modif->name()) + errorList.join(""));
+    
+    // At this point we can delete the modif template.
+    delete modif;
+    
     populateModifAndModifiedMonomerLists();
   
     mp_editorWnd->updateWholeSequenceMasses(true);
@@ -826,13 +825,13 @@ namespace massXpert
 
 	// The modif(quintptr) pointer is the third string in the list.
 	quintptr pointerCastToInt = stringList.at(3).toInt(&ok);      
-	Modif *modif =(Modif *) pointerCastToInt;
+	Modif *modif = reinterpret_cast<Modif *>(pointerCastToInt);
 
 	// The monomer(quintptr) pointer is the fourth string in the
 	// list.
 	ok = false;
 	pointerCastToInt = stringList.at(4).toInt(&ok);      
-	Monomer *monomer =(Monomer *) pointerCastToInt;
+	Monomer *monomer = reinterpret_cast<Monomer *>(pointerCastToInt);
       
 	//Because the sequence might have changed since the moment
 	//this window was opened, we should make sure the monomer to
