@@ -491,8 +491,8 @@ namespace massXpert
 			      }
 			    
 
-			    qDebug() << __FILE__ << __LINE__
-				     << "charge:" << oligomer->charge();
+// 			    qDebug() << __FILE__ << __LINE__
+// 				     << "charge:" << oligomer->charge();
 			
 			    oligomer->ionizeRule().debugPutStdErr();
 			
@@ -556,8 +556,8 @@ namespace massXpert
 						(totalCharge ? true : false),
 						-1, -1);
 
-				qDebug() << __FILE__ << __LINE__
-					 << "charge:" << oligomer->charge();
+// 				qDebug() << __FILE__ << __LINE__
+// 					 << "charge:" << oligomer->charge();
 				
 				oligomer->ionizeRule().debugPutStdErr();
 
@@ -588,10 +588,10 @@ namespace massXpert
 							    tempIonizeRule,
 							    false, -1, -1);
 				    
-				    qDebug() << __FILE__ << __LINE__
-					     << "charge:" 
-					     << static_cast<FragmentOligomer *>(oligomer)->
-				      charge();
+// 				    qDebug() << __FILE__ << __LINE__
+// 					     << "charge:" 
+// 					     << static_cast<FragmentOligomer *>(oligomer)->
+// 				      charge();
 
 				    oligomer->ionizeRule().debugPutStdErr();
 				    
@@ -638,10 +638,10 @@ namespace massXpert
 						    tempIonizeRule,
 						    true, -1, -1);
 				    
-				    qDebug() << __FILE__ << __LINE__
-					     << "charge:" 
-					     << static_cast<FragmentOligomer *>(oligomer)->
-				      charge();
+// 				    qDebug() << __FILE__ << __LINE__
+// 					     << "charge:" 
+// 					     << static_cast<FragmentOligomer *>(oligomer)->
+// 				      charge();
 				    
 				    oligomer->ionizeRule().debugPutStdErr();
 				    
