@@ -91,6 +91,7 @@ namespace massXpert
     return mp_oligomerList;
   }
 
+
   bool
   Cleaver::cleave(bool reset)
   {
@@ -105,7 +106,7 @@ namespace massXpert
     
     if (!m_cleaveOptions.motifList()->size())
       {
-	if(!m_cleaveOptions.parse())
+	if (!m_cleaveOptions.parse())
 	  {
 	    qDebug() << __FILE__ << __LINE__ 
 		     << "Failed to parse the cleavage options";
@@ -154,7 +155,7 @@ namespace massXpert
     
     for (int iter = 0 ; iter <= m_cleaveOptions.partials(); ++iter)
       {
-	if(cleavePartial(iter) == -1)
+	if (cleavePartial(iter) == -1)
 	  {
 	    qDebug() << __FILE__ << __LINE__ 
 		 << "Failed to perform partial cleavage at index:"
@@ -167,10 +168,10 @@ namespace massXpert
     // At this point we have the list of lists of oligomers, one list of
     // oligomers for each partial cleavage.
 
-    while(m_cleaveIndexList.size())
+    while (m_cleaveIndexList.size())
       m_cleaveIndexList.removeFirst();
   
-    while(m_noCleaveIndexList.size())
+    while (m_noCleaveIndexList.size())
       m_noCleaveIndexList.removeFirst();
   
     return true;
@@ -183,22 +184,32 @@ namespace massXpert
     QList<CleaveMotif *> *motifList = m_cleaveOptions.motifList();
   
 
-    while(m_cleaveIndexList.size())
+    while (m_cleaveIndexList.size())
       m_cleaveIndexList.removeFirst();
   
-    while(m_noCleaveIndexList.size())
+    while (m_noCleaveIndexList.size())
       m_noCleaveIndexList.removeFirst();
+
+    // Since version 2.3.0, the cleavage might be performed on a
+    // selected portion of a sequence only, not necessarily on the
+    // whole polymer sequence.
   
+    CoordinateList coordinateList = m_calcOptions.coordinateList();
+    Coordinates *coordinates = coordinateList.first();
+    
+    int startIndex = coordinates->start();
+    int endIndex = coordinates->end();
+
     for (int iter = 0; iter < motifList->size(); ++iter)
       {
 	CleaveMotif *cleaveMotif = motifList->at(iter);
 	Q_ASSERT(cleaveMotif);
       
-	int index = -1;
+	int index = startIndex -1;
       
 	while (1)
 	  {
-	    index = findCleaveMotif(*cleaveMotif, index + 1);
+	    index = findCleaveMotif(*cleaveMotif, index + 1, endIndex);
 	  
 	    if (index == -1)
 	      break;
@@ -223,7 +234,7 @@ namespace massXpert
 	    if (cleavageIndex < 0)
 	      continue;
 
-	    if (cleavageIndex >= mp_polymer->size() - 1)
+	    if (cleavageIndex >= endIndex)
 	      break;
 	  
 // 	    qDebug() << __FILE__ << __LINE__ 
@@ -235,7 +246,7 @@ namespace massXpert
 		m_cleaveIndexList.append(cleavageIndex);
 		
 // 		qDebug() << __FILE__ << __LINE__ 
-// 			  << "For cleavage";
+// 			  << "For cleavage, index:" << cleavageIndex;
 	      }
 	    else
 	      {
@@ -247,7 +258,7 @@ namespace massXpert
 	    
 	  }
 	// End of 
-	// while(1)
+	// while (1)
       }
     // End of
     // for (int iter = 0; iter < motifList->size(); ++iter)
@@ -287,7 +298,7 @@ namespace massXpert
 
 	int foundItemIndex = m_cleaveIndexList.indexOf(index, iter + 1);
 	
-	if(foundItemIndex != -1)
+	if (foundItemIndex != -1)
 	  {
 	    m_cleaveIndexList.removeAt(foundItemIndex);
 	    --iter;
@@ -299,7 +310,8 @@ namespace massXpert
   
   
   int 
-  Cleaver::findCleaveMotif(CleaveMotif &cleaveMotif, int index)
+  Cleaver::findCleaveMotif(CleaveMotif &cleaveMotif, 
+			   int startIndex, int endIndex)
   {
     bool noGood = false;
 
@@ -334,10 +346,10 @@ namespace massXpert
     if (codeList.size() == 0)
       return -1;
   
-    if (index < 0)
+    if (startIndex < 0)
       return -1;
     
-    if (index >= mp_polymer->size())
+    if (endIndex >= mp_polymer->size())
       return -1;
         
     // Seed the routine by setting 'first' to the first motif in the
@@ -349,9 +361,9 @@ namespace massXpert
     // sequence's list of monomers in search for a monomer having the
     // proper code ("Lys").
   
-    int iterIndex = index;
+    int iterIndex = startIndex;
  
-    while (iterIndex < monomerList.size())
+    while (iterIndex < endIndex)
       {
 	const Monomer *monomer = monomerList.at(iterIndex);
 
@@ -378,7 +390,7 @@ namespace massXpert
   
 	for (int iter = 1; iter < codeList.size(); ++iter)
 	  {
-	    if (iterIndex + iter >= mp_polymer->size())
+	    if (iterIndex + iter >= endIndex)
 	      {
 		noGood = true;
 		break;
@@ -418,7 +430,7 @@ namespace massXpert
 
   bool 
   Cleaver::accountCleaveRule(CleaveRule *cleaveRule, 
-			      CleaveOligomer *oligomer)
+			     CleaveOligomer *oligomer)
   {
     Q_ASSERT(cleaveRule);
     Q_ASSERT(oligomer);
@@ -439,7 +451,7 @@ namespace massXpert
 	Coordinates *coordinates = 
 	  static_cast<CoordinateList *>(oligomer)->at(iter);
 	
-	if(!cleaveRule->leftCode().isEmpty())
+	if (!cleaveRule->leftCode().isEmpty())
 	  {
 	    // The formula has to be there.
 	    
@@ -460,7 +472,7 @@ namespace massXpert
 		// there was no cleavage on this monomer, thus no rule
 		// to apply.
 		
-		if(!coordinates->start())
+		if (!coordinates->start())
 		  {
 		    // The monomer is not the left-end monomer, so the
 		    // match is real. Account for the formula !
@@ -471,7 +483,7 @@ namespace massXpert
 	      }
 	  }
   
-	if(!cleaveRule->rightCode().isEmpty())
+	if (!cleaveRule->rightCode().isEmpty())
 	  {
 	    // The formula has to be there.
       
@@ -492,7 +504,7 @@ namespace massXpert
 		// there was no cleavage on this monomer, thus no rule
 		// to apply.
 
-		if(coordinates->end() != mp_polymer->size() - 1)
+		if (coordinates->end() != mp_polymer->size() - 1)
 		  {
 		    // The monomer is not the right-end monomer, so
 		    // the match is real. Account for the formula !
@@ -509,7 +521,7 @@ namespace massXpert
 
 
   int
-  Cleaver::cleavePartial(int partialCleavage)
+  Cleaver::cleavePartial(int partialCleavageValue)
   {
     bool oligomerIsPolymer = false;
   
@@ -518,14 +530,25 @@ namespace massXpert
     static int leftIndex = 0;
     static int rightIndex = 0;
   
-
-    Q_ASSERT(partialCleavage >= 0);
+    Q_ASSERT(partialCleavageValue >= 0);
 
     OligomerList partialOligomerList;
 
-    leftIndex = 0;
-    rightIndex = 0;
+    // Since version 2.3.0, the cleavage might be performed on a
+    // selected portion of a sequence only, not necessarily on the
+    // whole polymer sequence. We have to know these values because
+    // when we create oligomer we'll have to set the coordinates of
+    // the monomers correctly.
   
+    CoordinateList coordinateList = m_calcOptions.coordinateList();
+    Coordinates *coordinates = coordinateList.first();
+    
+    int startIndex = coordinates->start();
+    int endIndex = coordinates->end();
+
+    leftIndex = startIndex;
+    rightIndex = 0;
+
     // Iterate in the array of indices where the cleavages should occur.
     
     for (iter = 0; iter < m_cleaveIndexList.size(); ++iter)
@@ -539,21 +562,21 @@ namespace massXpert
 	// element: the index at which the methionine occurs in the
 	// polymer sequence(and there is a single one). Now, Imagine
 	// that we are asked to perform a cleavage with
-	// 'partialCleavage' of 2. The way we do it is that we fetch the
+	// 'partialCleavageValue' of 2. The way we do it is that we fetch the
 	// index in the list of cleavage indices(m_cleaveIndexList) two
 	// positions farther than the position we are iterating:
 
-	// int partCleave = iter + partialCleavage;
+	// int partCleave = iter + partialCleavageValue;
       
 	// Now, if m_cleaveIndexList contains a single element, asking
-	// for this m_cleaveIndexList.at(iter + partialCleavage) will
+	// for this m_cleaveIndexList.at(iter + partialCleavageValue) will
 	// go out of the boundaries of the list, since it has a single
-	// item and partialCleavage is 2. This is what we are willing to
+	// item and partialCleavageValue is 2. This is what we are willing to
 	// avoid.
       
-	int partCleave = iter + partialCleavage;
+	int partCleave = iter + partialCleavageValue;
       
-	if(partCleave >= m_cleaveIndexList.size())
+	if (partCleave >= m_cleaveIndexList.size())
 	  {
 	    if (iter == 0)
 	      oligomerIsPolymer = true;
@@ -564,14 +587,14 @@ namespace massXpert
 	rightIndex = m_cleaveIndexList.at(partCleave);
 
 	QString name = QString("%1#%2")
-	  .arg(partialCleavage)
+	  .arg(partialCleavageValue)
 	  .arg(iter + 1);
  
       	CleaveOligomer *oligomer = 
 	  new CleaveOligomer(mp_polymer, 
 			      name, m_cleaveOptions.name(),
 			      Ponderable(),
-			      leftIndex, rightIndex, partialCleavage);
+			      leftIndex, rightIndex, partialCleavageValue);
 	
 	partialOligomerList.append(oligomer);
 
@@ -584,7 +607,7 @@ namespace massXpert
     // what's its real left end index !
   
     if (oligomerIsPolymer)
-      leftIndex = 0;
+      leftIndex = startIndex;
     else
       leftIndex = m_cleaveIndexList.at(--iter) + 1;
   
@@ -597,34 +620,35 @@ namespace massXpert
     // Remove 1 because the oligomer is described using indices and not
     // positions.
   
-    rightIndex = mp_polymer->size() - 1;
+    rightIndex = endIndex;
   
     QString name = QString("%1#%2")
-      .arg(partialCleavage)
+      .arg(partialCleavageValue)
       .arg(iter + 1);
   
     CleaveOligomer *oligomer = 
       new CleaveOligomer(mp_polymer, 
 			  name, m_cleaveOptions.name(),
 			  Ponderable(),
-			  leftIndex, rightIndex, partialCleavage);
+			  leftIndex, rightIndex, partialCleavageValue);
   
     partialOligomerList.append(oligomer);
   
-
     // At this point all the skeleton oligomers have been computed for
-    // the given partialCleavage. We still have to perform the
+    // the given partialCleavageValue. We still have to perform the
     // cross-link analysis prior to both calculate the masses and
     // perform the ionization of all the generated oligomers. Note that
     // making cross-link analysis is only useful in case the cleavage is
-    // full(partialCleavage == 0).
+    // full(partialCleavageValue == 0).
 
-    if (!partialCleavage)
+    if (!partialCleavageValue)
       {
-	if(m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
+	if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
 	  {
 	    if (analyzeCrossLinks(&partialOligomerList) == -1)
-	      return false;  
+	      {
+		return false;  
+	      }
 	  }
       }
     
@@ -639,7 +663,7 @@ namespace massXpert
 
     OligomerList waitOligomerList;
 
-    while(partialOligomerList.size())
+    while (partialOligomerList.size())
       {
 	CleaveOligomer *iterOligomer = 
 	  static_cast<CleaveOligomer *>(partialOligomerList.takeFirst());
@@ -721,7 +745,7 @@ namespace massXpert
     // At this point we should transfer all the oligomers from the
     // waitOligomerList to the initial oligomerList.
 
-    while(waitOligomerList.size())
+    while (waitOligomerList.size())
       {
 	Oligomer *iterOligomer = waitOligomerList.takeFirst();
       
@@ -733,7 +757,7 @@ namespace massXpert
 
     int oligomerCount = partialOligomerList.size();
     
-    while(partialOligomerList.size())
+    while (partialOligomerList.size())
       {
 	Oligomer *iterOligomer = partialOligomerList.takeFirst();
       
@@ -779,7 +803,7 @@ namespace massXpert
 	Oligomer *firstOligomer = 
 	  oligomerList->findOligomerEncompassing(firstMonomer, &foundIndex);
       
-	if(firstOligomer)
+	if (firstOligomer)
 	  {
 	    // At this point we should turn this oligomer into a
 	    // cross-linked oligomer, so that we can continue performing its
@@ -812,7 +836,7 @@ namespace massXpert
     // crossLinked oligomers from the crossLinkedOligomerList to
     // oligomerList.
 
-    while(crossLinkedOligomerList.size())
+    while (crossLinkedOligomerList.size())
       {
 	Oligomer *oligomer = crossLinkedOligomerList.takeAt(0);
       
@@ -856,9 +880,6 @@ namespace massXpert
       {
 	const Monomer *monomer = mp_polymer->at(iter);
 
-	//       qDebug() << __FILE__ << __LINE__ 
-	// 		<< monomer->name() << iter;
-      
 	// What crossLinks do involve this monomer ?
 
 	QList<int> crossLinkIndices;
@@ -867,7 +888,7 @@ namespace massXpert
 	  crossLinkList.crossLinksInvolvingMonomer(monomer,
 						    &crossLinkIndices);
       
-	if(ret)
+	if (ret)
 	  {
 	    // At least one cross-link involves the monomer currently
 	    // iterated in the oligomer being analysed.
@@ -890,7 +911,7 @@ namespace massXpert
 		// call below first checks that the cross-link is not
 		// already in the list).
 
-		if(!oligomer->addCrossLink(crossLink))
+		if (!oligomer->addCrossLink(crossLink))
 		  {
 		    // 		  qDebug() << __FILE__ << __LINE__ 
 		    // 			    << "The cross-link:" 
@@ -968,7 +989,7 @@ namespace massXpert
     //(this is what I call "pull-down" stuff). Found oligomers are
     // appended to the clearanceOligomerList.
 
-    while(clearanceOligomerList.size())
+    while (clearanceOligomerList.size())
       {
 	Oligomer *iterOligomer = clearanceOligomerList.first();
       
@@ -1044,7 +1065,7 @@ namespace massXpert
 			  oligomerList->findOligomerEncompassing(iterMonomer,
 								  &foundIndex);
 		  
-			if(foundOligomer)
+			if (foundOligomer)
 			  {
 			    // 			  qDebug() << __FILE__ << __LINE__ 
 			    // 				    << foundOligomer->name() << foundIndex;
@@ -1109,7 +1130,7 @@ namespace massXpert
   void 
   Cleaver::emptyOligomerList()
   {
-    while(!mp_oligomerList->isEmpty())
+    while (!mp_oligomerList->isEmpty())
       {
 	delete mp_oligomerList->first();
       }
