@@ -1632,4 +1632,90 @@ namespace massXpert
     return true;
   }
 
+  QStringList *
+  PolChemDef::differenceBetweenMonomers(double threshold, int monoOrAvg)
+  {
+//     qDebug() << __FILE__ << __LINE__
+// 	     << "threshold" << threshold;
+    
+    QStringList *list = new QStringList(); ;
+    
+    // We iterate in the list of monomers and compute a difference of
+    // mass for each monomer with respect to all the other ones. If
+    // the mass difference is less or equal to the threshold, then the
+    // pair is returned.
+
+    for (int iter = 0; iter < m_monomerList.size(); ++iter)
+      {
+	Monomer *monomer1 = m_monomerList.at(iter);
+	
+	monomer1->calculateMasses();
+	
+	for (int jter = 0; jter < m_monomerList.size(); ++jter)
+	  {
+	    // We are not going to explain that the same monomer has the
+	    // same mass !
+	    if (iter == jter)
+	      continue;
+	
+	    Monomer *monomer2 = m_monomerList.at(jter);
+	
+	    monomer2->calculateMasses();
+	
+	    // At this point we have the masses for both monomers.
+	
+	    if (monoOrAvg != MXT_MASS_MONO && monoOrAvg != MXT_MASS_AVG)
+	      {
+		qDebug() << __FILE__ << __LINE__
+			 << "Please set the mass type "
+		  "to value MXT_MASS_MONO or MXT_MASS_AVG";
+	    
+		return list;
+	      }
+	
+	    double diff = 0;
+	      
+	    if (monoOrAvg == MXT_MASS_MONO)
+	      {  
+		diff = monomer2->mass(MXT_MASS_MONO)
+		  - monomer1->mass(MXT_MASS_MONO);
+	      }
+	    else
+	      //(monoOrAvg == MXT_MASS_AVG)
+	      {
+		diff = monomer2->mass(MXT_MASS_AVG)
+		  - monomer1->mass(MXT_MASS_AVG);
+	      }
+
+	    // At this point, make sure that diff is within the
+	    // threshold. Note that to avoid duplicates, we remove all
+	    // values that are negative.
+	    
+	    if (diff >= 0 && diff <= threshold)
+	      {
+		QString line = QString("%1 - %2 = %3")
+		  .arg(monomer2->name())
+		  .arg(monomer1->name())
+		  .arg(diff);
+		
+		list->append(line);
+	      }
+	  }
+	// End of
+	// for (int jter = 0; jter < m_monomerList.size(); ++jter)
+      }
+    // End of
+    // for (int iter = 0; iter < m_monomerList.size(); ++iter)
+
+    // Only return a list if it contains at least one item.
+
+    if (!list->size())
+      {
+	delete list;
+	list = 0;
+      }
+    
+    return list;
+  }
+  
 } // namespace massXpert
