@@ -73,7 +73,29 @@ namespace massXpert
   }
   
 
-  //! Sets the code.
+  //! Sets the name.
+  /*!   
+    \param name The new name.
+  */
+  void 
+  Polymer::setName(const QString &name)
+  {
+    m_name = name;
+  }
+
+
+  //! Returns the name.
+  /*! 
+    \return The name.
+  */
+  QString
+  Polymer::name() const
+  {
+    return m_name;
+  }
+
+
+    //! Sets the code.
   /*!   
     \param code The new code.
   */
